

local MAJOR, MINOR = "LibPrint-1.0", 1

local LibPrint = LibStub:NewLibrary(MAJOR, MINOR)
if LibPrint == nil then return end


LibPrint.messageCache = {}
LibPrint.tabsAvailable = false

if GetChatWindowInfo(1) ~= "" then
	print("tab names are available")
	LibPrint.tabsAvailable = true
else
	print("tab names are not available")
end

local frame = CreateFrame("Frame")
function frame:OnEvent(...)
	print(...)
	print("tab names are available")
	LibPrint.tabsAvailable = true
	
	if table.maxn(LibPrint.messageCache) > 0 then
		print("Dumping cached messages")
		for i, data in ipairs(LibPrint.messageCache) do
			data.stream:Print(data.message)
		end
		print("Finished dumping cached messages")
	end
end
frame:SetScript("OnEvent", frame.OnEvent)
frame:RegisterEvent("UPDATE_CHAT_WINDOWS")


local Stream = {}
local stream = {}

Stream.prototype = stream


function LibPrint:NewStream(...)
	local stream = Stream:New()
	
	stream.destinations = { ... }
	return stream
end


function Stream:New()
	local instance = {}
	
	for k,v in pairs(self.prototype) do
		instance[k] = v
	end
	
	return instance
end


function stream:Print(...)
	--print("trying to print something")
	
	local message = self:BuildMessage(...)
	
	for i, destination in ipairs(self.destinations) do
		--print(destination, message)
		
		if type(destination) == "function" then
			destination(message)
			return
		elseif type(destination) == "string" then
			if not LibPrint.tabsAvailable then
				LibPrint:QueueMessage(message, self)
				return
			end
			
			--	look for the tab with this name
			for i=1,10 do
				local name, _, _, _, _, _, isShown, _, isDocked = GetChatWindowInfo(i)
				
				--	NOTE: FCF_OpenNewWindow
				
				if name == destination then
					--print("found frame match: (name, isShown, isDocked)")
					--print(name, isShown, isDocked)
				end
				
				if (isShown or isDocked) and name == destination then
					local frame = _G["ChatFrame"..i]
					frame:AddMessage(message)
					
					--	DEBUG
					--print("sending", "ChatFrame"..i, "this message:", message)
					return
				end
			end
		end
		
		--print("didn't find a destination")
	end
	
	--print("no destinations found")
end



function stream:BuildMessage(...)
	--	the vararg might end with nils, so count them 
	--	before packing
	local argCount = select("#", ...)
	
	if argCount < 1 then
		--	just ignore the call (don't even display prefix)
		return
	end

	local message = ""
	
	local units
	
	if self.prefix and self.prefix ~= "" then
		units = { self.prefix, ... }
		argCount = argCount+1
	else
		units = { ... }
	end

	if self.color then
		message = message.."|c"..self.color
	end
	
	--	print(nil, "s")
	--message = message..units[1]
	message = message.. (units[1] or "nil")
	
	for i=2,argCount do
		message = message .. " "
		
		if self.color then
			message = message .. "|c" .. self.color
		end
		
		message = message .. ( tostring(units[i]) or "nil" )
	end
	
	--	FIXME: don't break coloring after links

	return message
end


function LibPrint:QueueMessage(message, stream)
	if table.maxn(self.messageCache) == 0 then
		print("LibPrint: caching messages")
	end
	table.insert(self.messageCache, { message = message, stream = stream })
end
	


LibPrint:NewStream("LibPrint", "mcm", "debug", print):Print("loaded", MAJOR, "Revision", MINOR)


--	EXPERIMENTAL! SUBJECT TO CHANGE
function LibPrint:PrintPath()
	local stack = debugstack()
	local line = string.match(stack, "\n(.-)\n")
	--	FIXME: if path gets truncated...
	local shortPath = string.match(line, "Interface\\AddOns\\(.-):")	--	FIXME: other locales?
	print("LibPrint:", "running", shortPath or line)
end

LibPrint:PrintPath()


